from arango import ArangoClient
try:
    from arango.version import VERSION as arango_version
    using_v3 = True
except Exception as e:
    from arango.version import __version__ as arango_version
    using_v3 = False


class FlaskArangoDB(object):

    EXTENSION_NAME = 'flask_arangodb'

    def __init__(self, app=None, **kwargs):
        self.app = app
        self.arango = None

        if app is not None:
            self.init_app(app, **kwargs)

    def init_app(self, app, **kwargs):
        if not hasattr(app, 'extensions'):
            app.extensions = {}

        try:
            options = {
                'protocol': 'http',
                'host': app.config['ARANGO_HOST'],
                'port': app.config['ARANGO_PORT'],
                'db_name': app.config['ARANGO_DB'],
                'username': app.config['ARANGO_USERNAME'],
                'password': app.config['ARANGO_PASSWORD']
            }
            options.update(**kwargs)
            self.arango = ArangoDB(**options)

        except Exception as e:
            self.arango = None
            raise Exception('Unable to initialize ArangoDB ... {}'.format(e))

        app.extensions[self.EXTENSION_NAME] = self
        self.app = app

    @property
    def client(self):
        if self.arango is not None:
            return self.arango.client
        return None

    @property
    def db(self):
        if self.arango is not None:
            return self.arango.db
        return None


class ArangoDB(object):

    def __init__(self, host, port, db_name, username=None,
                 password=None, protocol='http'):
        self.host = host
        self.port = port
        self.db_name = db_name
        self.username = username
        self.password = password
        self.protocol = protocol
        self._client = None

    @property
    def client(self):
        if self._client is None:
            if using_v3:
                self._client = ArangoClient(
                    protocol=self.protocol,
                    host=self.host,
                    port=self.port,
                    username=self.username,
                    password=self.password,
                    enable_logging=True
                )
            else:
                self._client = ArangoClient(
                    protocol=self.protocol,
                    host=self.host,
                    port=self.port
                )
        return self._client

    @property
    def db(self):
        if self.client:
            if using_v3:
                return self.client.db(name=self.db_name)
            else:
                return self.client.db(name=self.db_name,
                                      username=self.username,
                                      password=self.password)



import re
import ast
import os
import sys
import shutil

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [
        ('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import coverage
        import pytest

        if self.pytest_args and len(self.pytest_args) > 0:
            self.test_args.extend(self.pytest_args.strip().split(' '))
            self.test_args.append('tests/')

        cov = coverage.Coverage()
        cov.start()
        errno = pytest.main(self.test_args)
        cov.stop()
        cov.report()
        cov.html_report()
        print("Wrote coverage report to htmlcov directory")
        sys.exit(errno)


do_cythonize = os.getenv('CYTHONIZE', 'false').lower() == 'true'
ext_modules = []
cmdclass = {'test': PyTest}
packages = find_packages(exclude=["tests"])

if do_cythonize:
    try:
        from Cython.Build import cythonize
        from Cython.Distutils import build_ext

        class MyBuildExt(build_ext):
            def run(self):
                build_ext.run(self)
                build_dir = os.path.realpath(self.build_lib)
                root_dir = os.path.dirname(os.path.realpath(__file__))
                target_dir = build_dir if not self.inplace else root_dir
                self.copy_file('rhodb/__init__.py', root_dir, target_dir)

            def copy_file(self, path, source_dir, destination_dir):
                if os.path.exists(os.path.join(source_dir, path)):
                    shutil.copyfile(os.path.join(source_dir, path),
                                    os.path.join(destination_dir, path))

        cmdclass['build_ext'] = MyBuildExt
        ext_modules = cythonize("rhodb/*.py")
        packages = []

    except ImportError:
        pass


_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('rhodb/__init__.py', 'rb') as f:
    __version__ = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))

setup(
    name='rhodb',
    version=__version__,
    description="Rho AI Database Helpers",
    long_description=open('README.md', 'r').read(),
    maintainer="Rho AI Corporation",
    license="Commercial",
    url="https://bitbucket.org/rhoai/rhodb",
    packages=packages,
    include_package_data=True,
    install_requires=[],
    ext_modules=ext_modules,
    extras_require={
        'sqlalchemy': [
            'psycopg2>=2.6,<2.8',
            'SQLAlchemy>=0.9.2,<1',
            'python-dateutil==2.4.2'
        ],
        'flask_sqlalchemy': [
            'psycopg2>=2.6,<2.8',
            'SQLAlchemy>=0.9.2,<1',
            'Flask>=0.10.1,<1.0',
            'Flask-SQLAlchemy>=2.1,<3.0',
            'python-dateutil==2.4.2'
        ],
        'redis': ['redis>=2.10.6,<4'],
        'cassandra': ['cassandra-driver>=3.1.1,<4'],
        'cassandra_redis': [
            'cassandra-driver>=3.1.1,<4',
            'redis>=2.10.6,<3'
        ],
        'aws': [
            'boto3>=1.2.3,<2'
        ],
        'elasticsearch': [
            'elasticsearch>=6.3,<7.0',
            'requests-aws4auth>=0.9,<1.0',
            'elasticsearch-dsl>=6.2,<7.0',
        ],
        'arangodb': [
            'python-arango >=3.12.1, < 5'
        ],
        'dev': [
            'honcho>=1.0,<2'
        ],
        'deploy': [
            'Cython>=0.29,<0.30',
            'wheel>=0.32,<0.33',
            'twine>=1.12,<2'
        ],
        'test': [
            'pytest==2.7.3',  # NOTE: Breaking changes in more recent pytest
            'tox>=3,<4',
            'coverage==4.0a5',
            'mock>=1,<2',
            'mysqlclient',
            'psycopg2>=2.6,<2.8',
            'redis>=2.9.1,<3'
        ]
    },
    cmdclass=cmdclass
)
